; Samsung S6B0755 LCD controller stuff


; --------------------------------
; Defines

PAGE_ADDRESS		equ	B'10110000'		; 1011xxxx, xxxx = address
COLUMN_ADDRESS_H	equ	B'00010000'		; last xxx = address
COLUMN_ADDRESS_L	equ	B'00000000'		; last xxxx = address
DISPLAY_ON			equ	B'10101111'
DISPLAY_OFF			equ	B'10101110'
ALL_BLACK			equ	B'10100101'		; 1010010x, x=1 forces all pixels on. 'Entire Display ON' in datasheet.
POWER_CONTROL		equ	B'00101111'		; all power circuits ON (LCD controlled)
DC_STEP_UP			equ	B'01100000'		; 011001xx, xx select 3-5x boosting
REGULATOR_RESISTOR	equ	B'00100001'		; 00100xxx (center)
SET_EVR_1			equ	B'10000001'		; 2-byte instruction!
SET_EVR_2			equ	B'00011100'		; XXxxxxxx, xxxxxx = value, XX = don't care. (center)
SET_DUTY_1			equ	B'01001000' 	; 2-byte instruction!
SET_DUTY_2			equ	B'01000000'
SET_BIAS			equ	B'01010001'		; 01010xxx, xxx = bias. Should be 001
DISPLAY_INVERT		equ	B'10100111'
DISPLAY_NOT_INVERT	equ	B'10100110'

SET_OSC_ON			equ	B'10101011'		; Starts internal oscillator
SET_POWER_SAVE		equ	B'10101000'		; 1010100x, x = power save level (0=standby, 1=sleep)
CLEAR_POWER_SAVE	equ	B'11100001'
SOFT_RESET			equ	B'11100010'		; Initialize the internal functions

SET_PAGE_ADDR		equ	B'10110000'		; 1011xxxx, xxxx = page addr.
SET_COLUMN_ADDR_1	equ	B'00010000'		; 2-byte! 00010xxx, xxx = Y[6..4]
SET_COLUMN_ADDR_2	equ	B'00000000'		; 0000xxxx, xxxx = Y[3..0]
; --------------------------------


lcd_reset:
	bcf		DATAPORT, RESETB
	call	delaysec
	bsf		DATAPORT, RESETB
	call	delaysec
	return

lcd_init:
	movlw	SOFT_RESET
	call	lcd_send_cmd

	call	delaysec

	movlw	SET_DUTY_1
	call	lcd_send_cmd
	call	delaysec
	movlw	SET_DUTY_2
	call	lcd_send_cmd
	call	delaysec
	movlw	SET_BIAS
	call	lcd_send_cmd
	call	delaysec
	; application setup finished...

	movlw	SET_OSC_ON
	call	lcd_send_cmd

	call	delaysec

	movlw	DC_STEP_UP
	call	lcd_send_cmd
	call	delaysec
	movlw	REGULATOR_RESISTOR
	call	lcd_send_cmd
	call	delaysec
	movlw	SET_EVR_1
	call	lcd_send_cmd
	call	delaysec
	movlw	SET_EVR_2
	call	lcd_send_cmd
	call	delaysec
	movlw	SET_BIAS
	call	lcd_send_cmd

	call	delaysec

	movlw	POWER_CONTROL
	call	lcd_send_cmd

	call	delaysec


;	movlw	ALL_BLACK		; for now, just see if we get anything at all...
;	call	lcd_send_cmd



	movlw	CLEAR_POWER_SAVE
	call	lcd_send_cmd

	call	delaysec

	movlw	DISPLAY_INVERT
	call	lcd_send_cmd
	call	delaysec

	movlw	DISPLAY_ON
	call	lcd_send_cmd
	call	delaysec

	return


; send the byte in WREG as a command
lcd_send_cmd:
	bcf		RSPORT, RS
	call	lcd_select
	call	sendSPI
	call	lcd_unselect
	bsf		RSPORT, RS
	return


; send the byte in WREG as pixel data
lcd_send_data:
	bsf		RSPORT, RS
	call	lcd_select
	call	sendSPI
	call	lcd_unselect
	return

lcd_select:
	bcf		CS1BPORT, CS1B
	return

lcd_unselect:
	bsf		CS1BPORT, CS1B
	return
