#include <p18cxxx.h>
#include <delays.h>
#include "system\typedefs.h"
#include "s6b0755.c"
#include "fontdata.c"
//#include "imagedata.c"
#include "io_cfg.h"


#define		SCAN_DIR		0x01	// LCD scan direction. 0x00 normal, 0x01 to flip upside down.
#define		BLACK_ON_WHITE	0x0	// 0 = white text on black, 1 = black text on white
#define		MAX_COLS		96
#define		MAX_ROWS		64
#define		MAX_PAGES		MAX_ROWS/8		// create compile-time constant of max. text rows (8-bit tall display pages)

void display_cr(void)	// carriage return
{
	if (SCAN_DIR==0x00)
	{
		lcd_send_cmd(SET_COLUMN_ADDR_1 | 0x00);	// high 3 bits of column addr
		lcd_send_cmd(SET_COLUMN_ADDR_2 | 0x00);	// low 4 bits
	}
	else
	{
		lcd_send_cmd(SET_COLUMN_ADDR_1 | 0x02);	// high 3 bits of column addr
		lcd_send_cmd(SET_COLUMN_ADDR_2 | 0x00);	// low 4 bits
	}
}

void display_home(void)
{
	display_cr();
	lcd_send_cmd(SET_PAGE_ADDR | 0x00);	// (cmd | 0x00) data r/w starts at page 0
}


void display_set_line(byte line)
{
	lcd_send_cmd(SET_PAGE_ADDR | (line&0x0F));	// (cmd | 0x00) data r/w starts at page 0
}

void display_set_col(byte col)
{
	if (SCAN_DIR==0x00)
	{
		lcd_send_cmd(SET_COLUMN_ADDR_1 | (col >> 4 & 0b00000111));	// high 3 bits of column addr
		lcd_send_cmd(SET_COLUMN_ADDR_2 | (col & 0x0F));	// low 4 bits
	}
	else
	{
		// FIXME: Backwardize
		col = col+32;
		lcd_send_cmd(SET_COLUMN_ADDR_1 | (col >> 4 & 0b00000111));	// high 3 bits of column addr
		lcd_send_cmd(SET_COLUMN_ADDR_2 | (col & 0x0F));	// low 4 bits
	}
}

void display_init(void)
{
	lcd_init();
	display_home();
	if (SCAN_DIR == 0)		// NOTE: MCC18 figures out correctly (this time) that SCAN_DIR is set at compile-time, so these may generate a "condition is always (true/false)" warning.
	{						// That doesn't stop it from truncating compile-time constants to unsigned short elsewhere though!
		lcd_send_cmd(SHL_SELECT | 0x00);	// normal COM direction
		lcd_send_cmd(ADC_SELECT | 0x00);	// normal SEG direction
	}
	else
	{
		lcd_send_cmd(SHL_SELECT | 0x08);	// reverse COM direction
		lcd_send_cmd(ADC_SELECT | 0x01);	// reverse SEG direction
	}

	//if (BLACK_ON_WHITE)
	{
		lcd_set_invert(BLACK_ON_WHITE);
	}

}

void display_off(void)
{
//	lcd_send_cmd(CMD_POWER_OFF, 1);
//	lcd_send_cmd(CMD_SET_OUTPUT | 0x07, 1);	// select case 06: 166 seg only
//	lcd_send_cmd(CMD_DUTY_PLUS | 0x00, 1);	// duty+1 mode off
//	lcd_send_cmd(CMD_LIGHT_ALL, 1);
}


void display_text(const rom char *addr)
{
	const rom char *charptr;
	byte line=0;
	byte temp=0;
	display_cr();

	while (*addr != 0 && line<MAX_COLS)
	{
		charptr = ((((unsigned int)*addr)-31) * 4) + charmap+2;	// WARNING: MCC18 performs this math at UInt8 level, resulting in an overflow, unless explicitly promoted.
		// NOTE: 'charmap' is the starting address of the character data, defined in fontdata.c. The first 31 ASCII junk characters have been omitted to save space, so subtract 31 to get the correct character number in the data table. Each entry is 4 bytes long, so multiply by 4 to get the starting index of that character's data. Finally, add 2 because the first 2 bytes specify the width and height of the character entries. If I weren't so lazy, I'd have actually used these instead of hardcoding them here.

		for (temp=0; temp<4; temp++)
		{
			lcd_send_data(*charptr++);
			line++;
		}
		*addr++;
	}

	while (line<MAX_COLS)					// clear the rest of the line
	{
		lcd_send_data(0x00);
		line++;
	}
}

void _user_putc(byte c)	// can use as user-defined stream destination
{
	byte line=0;
	byte temp=0;

	const rom char *charptr;

	if (c != 0 && line<MAX_COLS)
	{
		charptr = ((((unsigned int)c)-0x001F) * 4) + charmap+2;	// WARNING: MCC18 performs this math at UInt8 level, resulting in an overflow, unless explicitly promoted.
		for (temp=0; temp<4; temp++)
		{
			lcd_send_data(*charptr++);
			line++;
		}
	}
}

void display_pixmap(const rom char *addr, byte col, byte page) // NOTE: 'Page' is an 8px vertical boundary.
{

	const rom char *charptr;
	byte xsize=addr[0];
	byte ysize=addr[1];
	byte xx=0;
	byte yy=0;

	charptr=addr+2;

	for (yy=0; yy<ysize/8; yy++)
	{
		// handle pages
		display_set_line(yy+page);
		display_set_col(col);

		for (xx=0; xx<xsize; xx++)
		{
			//handle columns
			lcd_send_data(*charptr++);
		}
	}
}

void display_clear(void)
{
	byte i,j;
	display_home();
	for (i=0; i<MAX_PAGES; i++)
	{
		display_set_line(i);
		display_cr();
		for(j=0; j<MAX_COLS; j++)
		{
			lcd_send_data(0x00);
		}
	}
	display_home();
}


