/* $Id: s6b0755.c,v 1.1 2007/09/25 20:37:17 tgipson Exp $ */

#include <p18cxxx.h>
#include "system\typedefs.h"
#include "s6b0755.h"
#include "io_cfg.h"

#define		REGULATOR_RESISTOR_VAL	0x01	// range 0x00 ~ 0x03
#define		VOLUME_CONTROL_VAL		0x1C	// range 0x00 ~ 0x3F
#define		DISPLAY_INVERT_VAL		0x01	// range 0x00 ~ 0x01

void lcd_reset(void)
{
	LCD_RESETB=0;
	LCD_CS1B=1;
	LCD_SCK=1;
	LCD_SID=1;

	Delay100TCYx(1);

	LCD_RESETB=1;
}

// Bitbang LCD serial data using I/O lines. It would be much nicer to use actual serial lines and just WriteSPI(data);....
void lcd_write(byte data)	// FIXME: this is slow, write in ASM
{
		// NOTE: serial clk and data idle high
	byte i;
	LCD_SCK = 1;
	LCD_SID = 1;
	Delay10TCYx(1);
	LCD_CS1B = 0;
	Delay10TCYx(1);
	for (i=0; i<8; i++)
	{
		LCD_SID = (data & 0b10000000)>>7;	// clock out MSBit of data

		LCD_SCK = 0;

		Delay10TCYx(1);

		LCD_SCK = 1;

		Delay10TCYx(1);

		data = data << 0x01;
	}
	LCD_SID = 1;		// idle high
	LCD_CS1B = 1;
}


void lcd_send_cmd(byte data)	// Register Select '0' indicates the data is a command byte
{
	LCD_RS = 0;
	lcd_write(data);
	LCD_RS = 1;
}

void lcd_send_data(byte data)	// Register Select '1' indicates the data is a pixel data byte
{
	LCD_RS = 1;
	lcd_write(data);
}




void lcd_init(void)
{
	mInitLCD();
	lcd_reset();



	lcd_send_cmd(SOFT_RESET);
	Delay100TCYx(1);

	lcd_send_cmd(SET_DUTY_1);
	lcd_send_cmd(SET_DUTY_2);
	Delay100TCYx(1);

	lcd_send_cmd(SET_BIAS);
	Delay100TCYx(1);

	// application setup finished...

	lcd_send_cmd(SET_OSC_ON);
	Delay100TCYx(1);

	lcd_send_cmd(DC_STEP_UP);
	Delay100TCYx(1);

	lcd_send_cmd(REGULATOR_RESISTOR | REGULATOR_RESISTOR_VAL); // resistor value '001' @ 3.3V and 3x boost seems to work nicely
	Delay100TCYx(1);

	lcd_send_cmd(SET_EVR_1);
	lcd_send_cmd(SET_EVR_2 | VOLUME_CONTROL_VAL);
	Delay100TCYx(1);

	lcd_send_cmd(SET_BIAS);
	Delay100TCYx(1);

	lcd_send_cmd(POWER_CONTROL);
	Delay100TCYx(1);

	lcd_send_cmd(CLEAR_POWER_SAVE);
	Delay100TCYx(1);

	lcd_send_cmd(DISPLAY_INVERT | DISPLAY_INVERT_VAL);
	Delay100TCYx(1);

	lcd_send_cmd(DISPLAY_ON);
	Delay100TCYx(1);
}


void lcd_set_invert(byte val)
{
	lcd_send_cmd(DISPLAY_INVERT | (DISPLAY_INVERT_VAL & val));
}






/* ---------------------------------------------- */

